﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using RhythmBullet.Zer01HD.Game;
using RhythmBullet.Zer01HD.Game.ContentResolvers;
using RhythmBullet.Zer01HD.Game.Preferences;
using RhythmBullet.Zer01HD.UI;
using RhythmBullet.Zer01HD.Utilities;
using RhythmBullet.Zer01HD.Utilities.ContentSystem;
using RhythmBullet.Zer01HD.Utilities.DataTypes;
using RhythmBullet.Zer01HD.Utilities.UI;
using System;
using System.Diagnostics;

namespace RhythmBullet
{
    /// <summary>
    /// This is the main type for your game.
    /// </summary>
    public class RhythmBulletGame : Game
    {
        public const int WORLD_WIDTH = 4;
        public const int WORLD_HEIGHT = 3;
        public static int pixels_per_WU;

        public GraphicsDeviceManager Graphics;
        SpriteBatch spriteBatch;
        public readonly ContentSystem Assets;
        readonly ResolutionContentResolver resolutionContentResolver;
        public PreferencesManager preferencesManager;
        private Screen currentScreen;

        public RhythmBulletGame()
        {
            Graphics = new GraphicsDeviceManager(this);
            Graphics.PreferredBackBufferWidth = 320;
            Graphics.PreferredBackBufferHeight = 320;
            Window.IsBorderless = true;
            Content.RootDirectory = "Content";
            Assets = new ContentSystem(Content);
            resolutionContentResolver = new ResolutionContentResolver();
            FontContentResolver fcr = new FontContentResolver(resolutionContentResolver);
            Assets.contentPathModifier.Add(typeof(Texture2D), resolutionContentResolver);
            Assets.contentPathModifier.Add(typeof(SpriteFont), fcr);
            preferencesManager = new PreferencesManager(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + "/RhythmBullet",
                new General(),
                new Controls());
        }

        public Screen Screen
        {
            get
            {
                return currentScreen;
            }
            set
            {
                if (currentScreen != null)
                {
                    Screen.Hide();
                }
                currentScreen = value;
                Screen.Show();
            }
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            //Load preferences
            preferencesManager.Load();

            Resolution resolution = preferencesManager.GetPreferences<General>().Resolution;
            resolutionContentResolver.Width = resolution.Width;
            resolutionContentResolver.Height = resolution.Height;
            QueueContent();
            Debug.WriteLine("Initial setup complete.");
            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            Screen = new LoadingScreen(Content.Load<Texture2D>("recrown"));
            // TODO: use this.Content to load your game content here
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// game-specific content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.   
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            if (!Assets.Done)
            {
                Assets.Update();
            }
            Screen.Update(gameTime);

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.White);
            spriteBatch.Begin();
            Screen.Draw(spriteBatch);
            spriteBatch.End();

            base.Draw(gameTime);
        }

        public void PreAssetLoad()
        {
            Assets.UnloadAll();
            resolutionContentResolver.Width = Graphics.PreferredBackBufferWidth;
            resolutionContentResolver.Height = Graphics.PreferredBackBufferHeight;
            QueueContent();
            Screen.AssetLoadStateChange(true);
        }

        public void PostAssetLoad()
        {
            Screen.AssetLoadStateChange(false);
        }

        void QueueContent()
        {
            Assets.Queue<Texture2D>("Tech-Circle1");
            Assets.Queue<Texture2D>("polyjet-standard");
            Assets.Queue<Texture2D>("cybercircle3B");
            Assets.Queue<Texture2D>("title");
            Assets.Queue<Texture2D>("cybercircle1");
            Assets.Queue<Texture2D>("defaultCover");
            Assets.Queue<Texture2D>("laser");
            Assets.Queue<Texture2D>("pellet");
            Assets.Queue<Texture2D>("shard");
            Assets.Queue<Texture2D>("bar");
            Assets.Queue<Texture2D>("flake");
            Assets.Queue<Texture2D>("void_circle");
            Assets.Queue<Texture2D>("tpSelector");
            Assets.Queue<Texture2D>("backgrounds/mainBG");
            Assets.Queue<Texture2D>("magic1");
        }
    }
}
